/*
 * Sneaky-Seq - Phrase Sequencer Based on TK's Clockbox (MIDI Clock Sender)
 *
 * ==========================================================================
 *
 * Copyright (C) 2005  Thorsten Klose (tk@midibox.org)
 * 
 * ==========================================================================
 * 
 * This file is part of a MIOS application
 *
 * This application is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with This application; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ==========================================================================
 */

/////////////////////////////////////////////////////////////////////////////
// Include files
/////////////////////////////////////////////////////////////////////////////

#include "cmios.h"
#include "pic18f452.h"

#include "main.h"
#include "mclock.h"

/////////////////////////////////////////////////////////////////////////////
// Global variables
/////////////////////////////////////////////////////////////////////////////

mclock_state_t mclock_state; // the mclock state variable
mclock_pin_state_t mclock_pin_state; // state of the clock output pin

unsigned char mclock_tick_ctr; // requests MIDI clocks
unsigned char bpm; // holds the current BPM setting

unsigned char song; // song number that's currently loaded 0..50
unsigned char songload; // song number that's about to be loaded 0..50

unsigned char mclock_ctr_songdisplay;       // song number to be displayed 0..50 (not used yet)

unsigned char mclock_ctr_track1;       // track1 status 0=muted, 1=on
unsigned char mclock_ctr_track2;       // track2 status 0=muted, 1=on
unsigned char mclock_ctr_track3;       // track3 status 0=muted, 1=on
unsigned char mclock_ctr_track4;       // track4 status 0=muted, 1=on

unsigned char mclock_ctr_notetrack1; // current note on track 1
unsigned char mclock_ctr_notetrack2; // current note on track 2
unsigned char mclock_ctr_notetrack3; // current note on track 3
unsigned char mclock_ctr_notetrack4; // current note on track 4

unsigned char mclock_ctr_24;       // counts from 0..23
unsigned char mclock_ctr_beats;    // counts the quarter notes 0..3
unsigned char mclock_ctr_measures; // counts the measures (up to 65535)
unsigned char mclock_ctr_section; // section number (up to 15)
unsigned char mclock_ctr_seclength; // section length (up to 255)

unsigned char mclock_ctr_newsection; // upcoming section number (up to 15)
unsigned char mclock_ctr_changesection; // 1=load new section (0..1)
unsigned char mclock_ctr_resetmeasures; // 1=reset measures after section change (0..1)
unsigned char mclock_ctr_resettrigger; // 1=trigger note TX after restart (0..1)

unsigned char mclock_ctr_loadseq[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // Buffer array for loaded sequence 

unsigned char i;


/////////////////////////////////////////////////////////////////////////////
// Local variables
/////////////////////////////////////////////////////////////////////////////




// Here are my sequencer arrays: 
// {bpm, midi PC, number of sections, number of tracks, section length 1 - 16 }


const char sq01[20]={120,51,14,2,8,8,16,16,16,8,8,8,32,16,16,16,16,8,0,0};
const char sq02[20]={120,06,14,2,16,4,8,8,16,16,16,8,16,16,16,16,8,8,0,0};
const char sq03[20]={120,44,12,2,4,24,16,8,8,16,16,4,32,8,8,8,0,0,0,0};
const char sq04[20]={120,37,14,2,8,8,16,8,8,16,24,8,24,16,8,16,8,8,0,0};
const char sq05[20]={120,26,11,2,16,16,16,16,16,16,16,16,16,16,16,0,0,0,0,0};
const char sq06[20]={121,0,15,2,8,8,16,12,16,16,16,16,16,8,16,24,16,8,8,0};
const char sq07[20]={122,54,14,2,8,16,8,16,24,8,16,16,16,16,16,16,8,4,0,0};
const char sq08[20]={122,49,13,2,4,24,16,16,16,16,16,16,16,16,16,16,8,0,0,0};
const char sq09[20]={122,24,13,3,8,8,24,8,8,24,16,16,16,24,24,16,16,0,0,0};
const char sq10[20]={122,35,11,2,8,8,8,16,8,16,16,8,32,8,16,0,0,0,0,0};

const char sq11[20]={122,12,11,2,8,32,16,16,32,8,16,16,16,8,16,0,0,0,0,0};
const char sq12[20]={122,11,14,3,8,16,16,16,15,25,16,16,16,16,16,16,24,24,0,0};
const char sq13[20]={125,30,13,2,8,16,16,8,16,16,16,16,16,16,16,16,16,0,0,0};
const char sq14[20]={125,50,14,3,8,8,8,16,8,8,24,16,16,32,16,16,16,8,0,0};
const char sq15[20]={125,52,16,3,8,8,16,16,8,8,16,16,8,16,16,16,16,16,20,4};
const char sq16[20]={125,33,13,2,4,8,8,16,16,16,24,24,24,16,32,8,8,0,0,0};
const char sq17[20]={125,8,16,3,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,16};
const char sq18[20]={125,29,14,2,4,4,16,8,16,8,24,8,32,16,8,16,16,8,0,0};
const char sq19[20]={125,36,13,3,16,16,16,16,16,16,16,16,16,16,16,8,8,0,0,0};
const char sq20[20]={125,2,15,2,16,8,16,8,16,8,16,12,8,16,16,16,16,8,16,0};

const char sq21[20]={125,14,14,2,8,32,16,16,32,16,16,16,16,17,8,16,16,8,0,0};
const char sq22[20]={125,20,12,2,8,8,16,20,20,20,20,20,10,26,16,16,0,0,0,0};
const char sq23[20]={125,21,16,3,8,8,8,16,16,16,16,8,8,8,8,16,16,16,8,16};
const char sq24[20]={125,40,13,2,4,16,24,16,20,16,20,16,16,16,16,16,8,0,0,0};
const char sq25[20]={125,18,12,2,16,32,16,16,16,16,16,16,16,16,8,8,0,0,0,0};

const char sq26[20]={126,23,13,2,32,16,16,8,16,16,16,8,16,8,32,16,16,0,0,0};
const char sq27[20]={126,1,13,2,8,16,32,16,16,16,16,16,16,16,16,16,8,0,0,0};
const char sq28[20]={126,13,12,2,16,16,16,16,16,16,16,16,16,16,8,16,0,0,0,0};
const char sq29[20]={127,53,14,3,8,16,32,8,8,16,16,16,16,16,16,16,16,4,0,0};
const char sq30[20]={128,16,11,2,16,24,20,12,16,16,16,8,28,16,16,0,0,0,0,0};

const char sq31[20]={128,56,14,1,8,40,8,40,52,16,8,4,8,16,32,16,16,4,0,0};
const char sq32[20]={128,32,12,2,8,16,24,16,24,16,8,8,8,32,16,8,0,0,0,0};
const char sq33[20]={128,28,13,2,4,8,8,8,16,16,16,16,8,8,32,8,16,0,0,0};
const char sq34[20]={129,7,12,2,8,8,32,16,16,32,16,16,16,16,16,8,0,0,0,0};
const char sq35[20]={130,5,15,3,4,8,8,16,8,8,16,4,16,16,16,16,16,8,8,0};
const char sq36[20]={130,41,11,2,8,8,16,60,16,16,24,20,16,32,16,0,0,0,0,0};
const char sq37[20]={130,17,16,2,1,8,8,8,8,16,16,16,16,16,8,8,16,32,16,16};
const char sq38[20]={130,38,13,3,8,16,16,16,16,32,16,16,16,16,16,16,16,0,0,0};
const char sq39[20]={130,4,14,3,8,24,16,8,8,8,8,32,8,8,16,16,16,8,0,0};
const char sq40[20]={130,34,15,2,8,16,16,16,16,16,16,16,16,16,16,16,16,16,16,0};

const char sq41[20]={130,25,14,2,4,16,16,24,16,16,16,16,16,16,16,16,16,16,0,0};
const char sq42[20]={131,22,15,2,8,16,16,16,16,8,16,16,16,16,8,16,16,16,16,0};
const char sq43[20]={131,43,10,2,16,16,32,16,16,24,40,32,8,8,0,0,0,0,0,0};
const char sq44[20]={131,19,15,2,16,16,16,16,16,16,16,24,16,16,16,16,16,16,8,0};
const char sq45[20]={132,7,14,2,8,8,8,8,8,24,32,32,8,32,32,32,16,8,0,0};
const char sq46[20]={132,42,13,2,8,16,24,16,24,16,16,16,24,16,16,16,8,0,0,0};
const char sq47[20]={134,57,15,1,8,16,16,16,16,16,16,8,16,16,16,16,16,16,4,0};



/////////////////////////////////////////////////////////////////////////////
// This function initializes the MIDI clock module
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_Init(void)
{
  mclock_state.ALL = 0;
  mclock_pin_state.ALL = 0;
  mclock_tick_ctr = 0;

  mclock_ctr_newsection = 0;
  mclock_ctr_section = 0;
  song = 0;
  mclock_ctr_songdisplay = 0;
  mclock_ctr_seclength= 1;
  mclock_ctr_track1 = 0;
  mclock_ctr_track2 = 0;
  mclock_ctr_track3 = 0;
  mclock_ctr_track4 = 0;
  mclock_ctr_changesection = 0;	

  MCLOCK_ResetMeter();


}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////
// This function should be called from USER_Tick to send MIDI clock and
// MIDI Clock Start/Stop/Continue
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_Tick(void)
{

  // start request? Send 0xfa and enter RUN mode
  if( mclock_state.START_REQ ) {
    mclock_state.START_REQ = 0;
    MIOS_MIDI_TxBufferPut(0xfa);
    mclock_state.RUN = 1;
   MCLOCK_PlayNotes(1); // Play notes at beginning of pattern
    // request display update
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }



  // stutter request? Send Oxfc, followed by 0xfa and enter RUN mode
  if( mclock_state.STUTTER_REQ ) {
    mclock_state.STUTTER_REQ = 0;
    MIOS_MIDI_TxBufferPut(0xfc);
    MIOS_MIDI_TxBufferPut(0xfa);
    mclock_state.RUN = 1;
   MCLOCK_PlayNotes(1); // Play notes at beginning of pattern
    // request display update
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }



  // continue request? Send 0xfb and release pause
  if( mclock_state.CONT_REQ ) {
    mclock_state.CONT_REQ = 0;
    MIOS_MIDI_TxBufferPut(0xfb);
    mclock_state.PAUSE = 0;

    // request display update
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  // stop request? Send 0xfc and leave  RUN mode
  if( mclock_state.STOP_REQ ) {
    mclock_state.STOP_REQ = 0;
    MIOS_MIDI_TxBufferPut(0xfc);
    mclock_state.RUN = 0;
    mclock_state.PAUSE = 0;
    mclock_tick_ctr = 0; // prevent that any requested 0xf8 will be sent

    // request display update
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }

  // send 0xf8 until counter is 0
  while( mclock_tick_ctr ) {
    MIOS_MIDI_TxBufferPut(0xf8);

    // decrementing the counter *MUST* be an atomic operation, otherwise it could
    // conflict with MCLOCK_Timer()
    // however, I guess that the compiler will generate a single decf instruction,
    // which is atomic... but better to write it in this way, who knows, how SDCC
    // will behave in future...
    INTCONbits.GIE = 0; // disable interrupts
    --mclock_tick_ctr;
    INTCONbits.GIE = 1; // enable interrupts

    // increment the meter counters
    MCLOCK_PlayNotes(0);  // Play notes at beginning of pattern



	// Send a quarter note timing pulse for Kontakt Script

        if (mclock_ctr_24 == 0) {

		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 
		  MIOS_MIDI_TxBufferPut(1);//play note
		  MIOS_MIDI_TxBufferPut(0x01);//velocity 001
          }

        if( mclock_ctr_24 == 18 ) {

		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 
		  MIOS_MIDI_TxBufferPut(0);//play note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)
          }

    if( ++mclock_ctr_24 == 24 ) {
      mclock_ctr_24 = 0;

//      if(( ++mclock_ctr_beats == 4 ) || (mclock_ctr_resetmeasures == 1)) {

      if( ++mclock_ctr_beats == 4 ) {
	mclock_ctr_beats = 0;
	++mclock_ctr_measures;}

      if( mclock_ctr_measures == mclock_ctr_seclength) {
	mclock_ctr_measures = 0;
        mclock_ctr_changesection = 1;
      }


      


    }


    // request display update
    app_flags.DISPLAY_UPDATE_REQ = 1;
  }

}



/////////////////////////////////////////////////////////////////////////////
// This function loads a stored sequence into the current song
/////////////////////////////////////////////////////////////////////////////

void MCLOCK_LoadSeq(unsigned char _songload)
{

  // Load sequence depending on song number 
  // The ugliest bit of coding you'll ever see! haha

  switch(_songload) {

   case 0:
   	dosq01();
   break;

   case 1:
   	dosq02();
   break;

   case 2:
   	dosq03();
   break;

   case 3:
   	dosq04();
   break;

   case 4:
   	dosq05();
   break;

   case 5:
   	dosq06();
   break;

   case 6:
   	dosq07();
   break;

   case 7:
   	dosq08();
   break;

   case 8:
   	dosq09();
   break;

   case 9:
   	dosq10();
   break;

   case 10:
   	dosq11();
   break;

   case 11:
   	dosq12();
   break;

   case 12:
   	dosq13();
   break;

   case 13:
   	dosq14();
   break;

   case 14:
   	dosq15();
   break;

   case 15:
   	dosq16();
   break;

   case 16:
   	dosq17();
   break;

   case 17:
   	dosq18();
   break;

   case 18:
   	dosq19();
   break;

   case 19:
   	dosq20();
   break;

   case 20:
   	dosq21();
   break;

   case 21:
   	dosq22();
   break;

   case 22:
   	dosq23();
   break;

   case 23:
   	dosq24();
   break;

   case 24:
   	dosq25();
   break;

   case 25:
   	dosq26();
	 break;

   case 26:
   	dosq27();
   break;

   case 27:
   	dosq28();
   break;

   case 28:
   	dosq29();
   break;

   case 29:
   	dosq30();
   break;

   case 30:
   	dosq31();
   break;

   case 31:
   	dosq32();
   break;

   case 32:
   	dosq33();
   break;

   case 33:
   	dosq34();
   break;

   case 34:
   	dosq35();
   break;

   case 35:
   	dosq36();
   break;

   case 36:
   	dosq37();
   break;

   case 37:
   	dosq38();
   break;

   case 38:
   	dosq39();
   break;

   case 39:
   	dosq40();
   break;

   case 40:
   	dosq41();
   break;

   case 41:
   	dosq42();
   break;

   case 42:
   	dosq43();
   break;

   case 43:
   	dosq44();
   break;

   case 44:
   	dosq45();
   break;

   case 45:
   	dosq46();
   break;

   case 46:
   	dosq47();
   break;

  }
  

  // Reset Section LED's 
  MIOS_DOUT_PinSet((mclock_ctr_loadseq[2]-1),1); 

  // Send corresponding midi PC for newly loaded song

  MIOS_MIDI_TxBufferPut(0xc0);//Initiate PC on Channel #1
  MIOS_MIDI_TxBufferPut(mclock_ctr_loadseq[1]);// PC loaded from sequences


  // New BPM to midi CC subrouting - this communicates the current tempo 
  // info to my NI Kontakt scripts

               MIOS_MIDI_TxBufferPut(0xb0);           // CC at MIDI Channel #1
               MIOS_MIDI_TxBufferPut(0x77);           // CC# 119 (reserved)
               MIOS_MIDI_TxBufferPut(bpm - 80);


  
}


/////////////////////////////////////////////////////////////////////////////
// This function plays the notes for all tracks 
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_PlayNotes(unsigned char _trigger)
{


        // If a section has been changed or retriggered (mclock_ctr_resettrigger = 1)
        // or a new song has been loaded, then play the notes for each track
       

        mclock_ctr_resettrigger = _trigger;        

        // Only send notes at the beginning of the beat+measure
        if ((mclock_ctr_beats == 0) && (mclock_ctr_24 == 0)) {

        // Only send notes if mclock_ctr_resttrigger or changesection has been set

          if ((mclock_ctr_changesection == 1) || (mclock_ctr_resettrigger == 1)){
      
                mclock_ctr_seclength = mclock_ctr_loadseq[(mclock_ctr_section + 4)];

                 // play notes on track 1 if track 1 is unmuted, ie. = 1
                 if (mclock_ctr_track1 == 1){ 
 		  // NOTE OFF (previously played note)
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack1);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)

		  // PLAY NOTE
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_section+24);//play note
		  MIOS_MIDI_TxBufferPut(0x7f);//velocity 127

		  mclock_ctr_notetrack1 = mclock_ctr_section+24;
                  }


                 // play notes on track 2 if track 2 is unmuted, ie. = 1
                 if (mclock_ctr_track2 == 1){ 
 		  // NOTE OFF (previously played note)
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack2);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)

		  // PLAY NOTE
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_section+48);//play note
		  MIOS_MIDI_TxBufferPut(0x7f);//velocity 127

		  mclock_ctr_notetrack2 = mclock_ctr_section+48;
                  }


                 // play notes on track 3 if track 3 is unmuted, ie. = 1
                 if (mclock_ctr_track3 == 1){ 
 		  // NOTE OFF (previously played note)
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack3);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)

		  // PLAY NOTE
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_section+72);//play note
		  MIOS_MIDI_TxBufferPut(0x7f);//velocity 127

		  mclock_ctr_notetrack3 = mclock_ctr_section+72;
        
                  }

         // reset changesection trigger and reset measures to 0
        
         mclock_ctr_changesection = 0;	
         mclock_ctr_measures = 0;        
         app_flags.DISPLAY_UPDATE_REQ = 1; 

       }
      }



}




/////////////////////////////////////////////////////////////////////////////
// This function mutes and unmutes a track
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_DoTrackMute(unsigned char track)
{

 switch (track) {

   case 1:
           // if the track is to be muted, then send a note off and turn off the track light
           if (mclock_ctr_track1 == 1) {
 		  // NOTE OFF (previously played note)
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack1);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)
                  mclock_ctr_track1 = 0;
                  MIOS_DOUT_PinSet(16,0); 

                } else {

   		   // if the track is to be turned on, 
                   //then enable the track and turn on the track light

                  mclock_ctr_track1 = 1;
                  MIOS_DOUT_PinSet(16,1); 
                }
   break;

   case 2:
          if (mclock_ctr_loadseq[3] > 1) {
  
           // if the track is to be muted, then send a note off and turn off the track light
           if (mclock_ctr_track2 == 1) {
 		  // NOTE OFF (previously played note)
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack2);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)
                  mclock_ctr_track2 = 0;
                  MIOS_DOUT_PinSet(17,0); 
                } else {

   		   // if the track is to be turned on, 
                   //then enable the track and turn on the track light

                  mclock_ctr_track2 = 1;
                  MIOS_DOUT_PinSet(17,1); 

                }
               }
   break;

   case 3:
           // if the track is to be muted, then send a note off and turn off the track light
          if (mclock_ctr_loadseq[3] > 2) {
           if (mclock_ctr_track3 == 1) {
 		  // NOTE OFF (previously played note)
		  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack3);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)
                  mclock_ctr_track3 = 0;
                  MIOS_DOUT_PinSet(18,0); 
                } else {             

   		   // if the track is to be turned on, 
                   //then enable the track and turn on the track light

                  mclock_ctr_track3 = 1;
                  MIOS_DOUT_PinSet(18,1); 
                }
               }
   break;


   }
}

/////////////////////////////////////////////////////////////////////////////
// This function should be called from USER_Timer to update the MIDI clock
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_Timer(void)
{
  // just increment a clock counter if in RUN and not in PAUSE mode
  // it will be decremented in MCLOCK_Tick - each step sends one 0xf8 event
  // the advantage of this counter is, that a clock event can never get
  // lost, regardless how much the CPU is loaded
  // (however, in this application the CPU is very unbusy, but I prepared
  // this for even more complex programs...)

  if( mclock_state.RUN && !mclock_state.PAUSE) {
    ++mclock_tick_ctr;
    mclock_pin_state.CLK_REQ = 1;
  }
}

/////////////////////////////////////////////////////////////////////////////
// This internal function divides 3125000 / BPM
// The formula:
//   -> delay = 60 / BPM * 24
//   timer is clocked at 10 MHz, and we are using a 1:8 prescaler
//   -> timer cycles = ((60/BPM*24)/8) / 100E-9
//   -> 3125000 / BPM
//
// the 24 Bit / 16 Bit division routine has been created by Nikolai Golovchenko, 
// and is published at:
// http://www.piclist.org/techref/microchip/math/div/24by16.htm
/////////////////////////////////////////////////////////////////////////////
unsigned char AARGB0;
unsigned char AARGB1;
unsigned char AARGB2;
unsigned char BARGB0;
unsigned char BARGB1;
unsigned char LOOPCOUNT;
unsigned char REMB0;
unsigned char REMB1;
unsigned int MCLOCK_GetTimerValue(unsigned char bpm)
{

  //;Inputs:
  //;   Dividend - AARGB0:AARGB1:AARGB2 (0 - most significant!)
  //;   Divisor  - BARGB0:BARGB1
  //;Temporary:
  //;   Counter  - LOOPCOUNT
  //;   Remainder- REMB0:REMB1
  //;Output:
  //;   Quotient - AARGB0:AARGB1:AARGB2

  LOOPCOUNT = 24;
  AARGB0 = (unsigned char)(3125000L >> 16L);
  AARGB1 = (unsigned char)(3125000L >>  8L);
  AARGB2 = (unsigned char)(3125000L >>  0L);
  BARGB0 = 0;
  BARGB1 = bpm;
  REMB0 = 0;
  REMB0 = 1;

__asm
      banksel _LOOPCOUNT;

        CLRF _REMB0
        CLRF _REMB1
        MOVLW 24
        MOVWF _LOOPCOUNT
LOOPU2416:
        RLCF _AARGB2, W         ;shift dividend left to move next bit to remainder
        RLCF _AARGB1, F         ;
        RLCF _AARGB0, F         ;

        RLCF _REMB1, F          ;shift carry (next dividend bit) into remainder
        RLCF _REMB0, F

        RLCF _AARGB2, F         ;finish shifting the dividend and save  carry in AARGB2.0,
                                ;since remainder can be 17 bit long in some cases
                                ;(e.g. 0x800000/0xFFFF). This bit will also serve
                                ;as the next result bit.
         
        MOVF _BARGB1, W         ;substract divisor from 16-bit remainder
        SUBWF _REMB1, F         ;
        MOVF _BARGB0, W         ;
	SKPC                    ;
        INCFSZ _BARGB0, W       ;
        SUBWF _REMB0, F         ;

;here we also need to take into account the 17th bit of remainder, which
;is in AARGB2.0. If we dont have a borrow after subtracting from lower
;16 bits of remainder, then there is no borrow regardless of 17th bit 
;value. But, if we have the borrow, then that will depend on 17th bit 
;value. If it is 1, then no final borrow will occur. If it is 0, borrow
;will occur. These values match the borrow flag polarity.

        SKPNC                   ;if no borrow after 16 bit subtraction
         BSF _AARGB2, 0         ;then there is no borrow in result. Overwrite
                                ;AARGB2.0 with 1 to indicate no
                                ;borrow.
                                ;if borrow did occur, AARGB2.0 already
                                ;holds the final borrow value (0-borrow,
                                ;1-no borrow)

        BTFSC _AARGB2, 0        ;if no borrow after 17-bit subtraction
         BRA UOK46LL            ;skip remainder restoration.

        ADDWF _REMB0, F         ;restore higher byte of remainder. (w 
                                ;contains the value subtracted from it
                                ;previously)
        MOVF _BARGB1, W         ;restore lower byte of remainder
        ADDWF _REMB1, F         ;

UOK46LL:

        DECFSZ _LOOPCOUNT, f    ;decrement counter
         BRA LOOPU2416         ;and repeat the loop if not zero.

      movff _AARGB1, _PRODL
      movf  _AARGB2, W
      return
__endasm;
 return 0; // dummy return 
}



/////////////////////////////////////////////////////////////////////////////
// This function loads the new song and lights up the corresponding track mute
// LED's
/////////////////////////////////////////////////////////////////////////////

void MCLOCK_DoSongLoad(void)
{

unsigned char i,j;


 for (j = 0; j < 16; j++) {
    MIOS_DOUT_PinSet(j,0);
    }

    MCLOCK_LoadSeq(song);


   // start by turning off all track lights
   i = mclock_ctr_loadseq[3];

    MIOS_DOUT_PinSet(16,0);
    MIOS_DOUT_PinSet(17,0);
    MIOS_DOUT_PinSet(18,0);
    MIOS_DOUT_PinSet(19,0);

   // if the song has one track, turn on the first track light
   if (i == 1){
    mclock_ctr_track1 = 1;
    MIOS_DOUT_PinSet(16,1);
   }   

   // if the song has 2 tracks, turn on the first 2 track lights
   if (i == 2){
    mclock_ctr_track1 = 1;
    MIOS_DOUT_PinSet(16,1);
    mclock_ctr_track2 = 1;
    MIOS_DOUT_PinSet(17,1);
   }   

   // if the song has 3 tracks, turn on the first 3 track lights
   if (i == 3){
    mclock_ctr_track1 = 1;
    MIOS_DOUT_PinSet(16,1);
    mclock_ctr_track2 = 1;
    MIOS_DOUT_PinSet(17,1);
    mclock_ctr_track3 = 1;
    MIOS_DOUT_PinSet(18,1);
   }   

  
}



/////////////////////////////////////////////////////////////////////////////
// This function returns the current song number
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_SongNumberSet(unsigned char _songnumber)
{  
  song = _songnumber;
}

unsigned char MCLOCK_SongNumberGet(void)
{
  return song;
}





/////////////////////////////////////////////////////////////////////////////
// These functions are used to set/query the BPM
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_BPMSet(unsigned char _bpm)
{
  // re-init timer depending on new BPM value
  bpm = _bpm;
  MIOS_TIMER_ReInit(3, MCLOCK_GetTimerValue(bpm));

         /////////////////////////////////////////////////////////////////////////////
         // New BPM to midi CC subrouting - this communicates the current tempo 
         // info to my NI Kontakt scripts
         /////////////////////////////////////////////////////////////////////////////

               MIOS_MIDI_TxBufferPut(0xb0);           // CC at MIDI Channel #1
               MIOS_MIDI_TxBufferPut(0x77);           // CC# 119 (reserved)
               MIOS_MIDI_TxBufferPut(bpm - 80);

}

unsigned char MCLOCK_BPMGet(void)
{
  return bpm;
}




/////////////////////////////////////////////////////////////////////////////
// This function resets the mclock_ctr variables
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_ResetMeter(void)
{
  mclock_ctr_24 = 0;
  mclock_ctr_beats = 0;
  mclock_ctr_measures = 0;

}




/////////////////////////////////////////////////////////////////////////////
// This function sends the current song position
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_SendMeter(void)
{
  unsigned int songpos = (mclock_ctr_beats << 2) | (mclock_ctr_measures << 4);

  MIOS_MIDI_TxBufferPut(0xf2);
  MIOS_MIDI_TxBufferPut((unsigned char)(songpos & 0x7f));
  MIOS_MIDI_TxBufferPut((unsigned char)(songpos >> 7) & 0x7f);
}


/////////////////////////////////////////////////////////////////////////////
// This function sets the new section
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_DoSectionSet(unsigned char section) 
{

        mclock_ctr_newsection = section;
	mclock_ctr_changesection = 1;

        if ((mclock_ctr_section+1) != (mclock_ctr_loadseq[2])){ 

        // Turn off the LED that corresponds with the previous section
	 MIOS_DOUT_PinSet(mclock_ctr_section,0); 				
        }

        // Turn on the LED that corresponds with the current section
        mclock_ctr_section = section;
	MIOS_DOUT_PinSet(mclock_ctr_section,1); 				
	}



/////////////////////////////////////////////////////////////////////////////
// These functions are used to control the MCLOCK handler from external
/////////////////////////////////////////////////////////////////////////////
void MCLOCK_DoStop(void)
{
  if( mclock_state.RUN == 0 ) {
    // reset song position of already in stop mode
    MCLOCK_ResetMeter();
    // send Song Position
    MCLOCK_SendMeter();
  }
  // request stop
  mclock_state.STOP_REQ = 1;

	// All notes off
	  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack1);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)

	  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack2);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)

	  MIOS_MIDI_TxBufferPut(144 + 0);//Note On, channel 0 (+13)
		  MIOS_MIDI_TxBufferPut(mclock_ctr_notetrack3);//last played note
		  MIOS_MIDI_TxBufferPut(0x00);//velocity 0 (=> equ Note off)


  // request display update
  app_flags.DISPLAY_UPDATE_REQ = 1;
}

void MCLOCK_DoPause(void)
{
  // if in RUN mode:
  if( mclock_state.RUN ) {
    // toggle pause mode
    if( mclock_state.PAUSE ) {
      mclock_state.CONT_REQ = 1;
    } else {
      mclock_state.PAUSE = 1;
    }
  } else {
    // Stop mode: just toggle PAUSE
    mclock_state.PAUSE = mclock_state.PAUSE ? 0 : 1;
  }

  // request display update
  app_flags.DISPLAY_UPDATE_REQ = 1;
}

void MCLOCK_DoPlay(void)
{
  // reset meter counters
  MCLOCK_ResetMeter();
  // send Song Position
  MCLOCK_SendMeter();
  // request start
  mclock_state.START_REQ = 1;

         /////////////////////////////////////////////////////////////////////////////
         // New BPM to midi CC subrouting - this communicates the current tempo 
         // info to my NI Kontakt scripts
         /////////////////////////////////////////////////////////////////////////////

               //Transmit Tempo as midi CC 
               MIOS_MIDI_TxBufferPut(0xb0);           // CC at MIDI Channel #1
               MIOS_MIDI_TxBufferPut(0x77);           // CC# 119 (reserved)
               MIOS_MIDI_TxBufferPut(bpm - 80);


  // request display update
  app_flags.DISPLAY_UPDATE_REQ = 1;
}




// This function does a "stutter" effect - it stops the sequencer then starts it
// immediately after

void MCLOCK_DoStutter(void)
{

  // reset meter counters
  MCLOCK_ResetMeter();
  // send Song Position
  MCLOCK_SendMeter();
  // request start
  mclock_state.STUTTER_REQ = 1;

         /////////////////////////////////////////////////////////////////////////////
         // New BPM to midi CC subrouting - this communicates the current tempo 
         // info to my NI Kontakt scripts
         /////////////////////////////////////////////////////////////////////////////

               //Transmit Tempo as midi CC 
               MIOS_MIDI_TxBufferPut(0xb0);           // CC at MIDI Channel #1
               MIOS_MIDI_TxBufferPut(0x77);           // CC# 119 (reserved)
               MIOS_MIDI_TxBufferPut(bpm - 80);


  // request display update
  app_flags.DISPLAY_UPDATE_REQ = 1;
}






void MCLOCK_DoRew(void)
{
		  // decrement measure and reset subcounters
		//  if( mclock_ctr_measures ) --mclock_ctr_measures;
		//  mclock_ctr_24 = 0;
		//  mclock_ctr_beats = 0;

		  // send Song Position
		//  MCLOCK_SendMeter();

         /////////////////////////////////////////////////////////////////////////////
         // New BPM to midi CC subrouting - this communicates the current tempo 
         // info to my NI Kontakt scripts
         /////////////////////////////////////////////////////////////////////////////

               MIOS_MIDI_TxBufferPut(0xbf);           // CC at MIDI Channel #16
               MIOS_MIDI_TxBufferPut(0x77);           // CC# 119 (reserved)
               MIOS_MIDI_TxBufferPut(bpm - 80);



  // request display update
  app_flags.DISPLAY_UPDATE_REQ = 1;
}

void MCLOCK_DoFwd(void)
{
	  // increment measure and reset subcounters
		//  ++mclock_ctr_measures;
		//  mclock_ctr_24 = 0;
		//  mclock_ctr_beats = 0;

		  // send Song Position
		//  MCLOCK_SendMeter();

         /////////////////////////////////////////////////////////////////////////////
         // New BPM to midi CC subrouting - this communicates the current tempo 
         // info to my NI Kontakt scripts
         /////////////////////////////////////////////////////////////////////////////

               MIOS_MIDI_TxBufferPut(0xbf);           // CC at MIDI Channel #16
               MIOS_MIDI_TxBufferPut(0x77);           // CC# 119 (reserved)
               MIOS_MIDI_TxBufferPut(bpm - 80);


  // request display update
  app_flags.DISPLAY_UPDATE_REQ = 1;
}



  
void dosq01(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq01[i];  
    }
}  
void dosq02(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq02[i];  
    }
}  
void dosq03(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq03[i];  
    }
}  
void dosq04(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq04[i];  
    }
}  
void dosq05(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq05[i];  
    }
}  
void dosq06(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq06[i];  
    }
}  
void dosq07(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq07[i];  
    }
}  
void dosq08(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq08[i];  
    }
}  
void dosq09(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq09[i];  
    }
}  
void dosq10(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq10[i];  
    }
}    
void dosq11(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq11[i];  
    }
}  
void dosq12(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq12[i];  
    }
}  
void dosq13(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq13[i];  
    }
}  
void dosq14(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq14[i];  
    }
}  
void dosq15(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq15[i];  
    }
}  
void dosq16(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq16[i];  
    }
}  
void dosq17(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq17[i];  
    }
}  
void dosq18(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq18[i];  
    }
}  
void dosq19(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq19[i];  
    }
}  
void dosq20(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq20[i];  
    }
}    
void dosq21(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq21[i];  
    }
}  
void dosq22(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq22[i];  
    }
}  
void dosq23(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq23[i];  
    }
}  
void dosq24(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq24[i];  
    }
}  
void dosq25(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq25[i];  
    }
}  
void dosq26(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq26[i];  
    }
}  
void dosq27(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq27[i];  
    }
}  
void dosq28(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq28[i];  
    }
}  
void dosq29(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq29[i];  
    }
}  
void dosq30(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq30[i];  
    }
}    
void dosq31(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq31[i];  
    }
}  
void dosq32(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq32[i];  
    }
}  
void dosq33(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq33[i];  
    }
}  
void dosq34(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq34[i];  
    }
}  
void dosq35(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq35[i];  
    }
}  
void dosq36(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq36[i];  
    }
}  
void dosq37(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq37[i];  
    }
}  
void dosq38(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq38[i];  
    }
}  
void dosq39(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq39[i];  
    }
}  
void dosq40(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq40[i];  
    }
}  
void dosq41(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq41[i];  
    }
}  
void dosq42(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq42[i];  
    }
}  
void dosq43(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq43[i];  
    }
}  
void dosq44(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq44[i];  
    }
}  
void dosq45(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq45[i];  
    }
}  
void dosq46(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq46[i];  
    }
}  
void dosq47(void) {  
     for (i = 0; i < 20; i++) {
     mclock_ctr_loadseq[i] = sq47[i];  
    }
}  



