/*
 * Sneaky-Seq - Phrase Sequencer Based on TK's Clockbox (MIDI Clock Sender)
 *
 * MIDI clock module definitions
 *
 * ==========================================================================
 *
 * Copyright (C) 2005  Thorsten Klose (tk@midibox.org)
 * 
 * ==========================================================================
 * 
 * This file is part of a MIOS application
 *
 * This application is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with This application; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ==========================================================================
 */

#ifndef _MCLOCK_H
#define _MCLOCK_H

/////////////////////////////////////////////////////////////////////////////
// Global definitions
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// Global Types
/////////////////////////////////////////////////////////////////////////////

// status of midi clock
typedef union {
  struct {
    unsigned ALL:8;
  };
  struct {
    unsigned RUN:1;
    unsigned PAUSE:1;
    unsigned STUTTER_REQ:1;
    unsigned START_REQ:1;
    unsigned STOP_REQ:1;
    unsigned CONT_REQ:1;
  };
} mclock_state_t;

// status of clock output pin
typedef union {
  struct {
    unsigned ALL:8;
  };
  struct {
    unsigned CLK_REQ:1;
  };
} mclock_pin_state_t;


/////////////////////////////////////////////////////////////////////////////
// Prototypes
/////////////////////////////////////////////////////////////////////////////



void MCLOCK_Init(void);
void MCLOCK_Tick(void);
void MCLOCK_Timer(void);

void MCLOCK_BPMSet(unsigned char bpm);
unsigned char MCLOCK_BPMGet(void);

void MCLOCK_SongNumberSet(unsigned char songload);

unsigned char MCLOCK_SongNumberGet(void);

void MCLOCK_DoSongLoad(void);

void MCLOCK_DoTrackMute(unsigned char track);

void MCLOCK_ResetMeter(void);
void MCLOCK_SendMeter(void);


void MCLOCK_DoSectionReset(void);

void MCLOCK_DoSectionSet(unsigned char section);

void MCLOCK_PlayNotes(unsigned char trigger);
void MCLOCK_LoadSeq(unsigned char song);



void MCLOCK_DoStop(void);
void MCLOCK_DoStutter(void);
void MCLOCK_DoPause(void);
void MCLOCK_DoPlay(void);
void MCLOCK_DoRew(void);
void MCLOCK_DoFwd(void);

unsigned int MCLOCK_GetTimerValue(unsigned char bpm);

/////////////////////////////////////////////////////////////////////////////
// Export global variables
/////////////////////////////////////////////////////////////////////////////

extern mclock_state_t mclock_state;
extern mclock_pin_state_t mclock_pin_state;

extern unsigned char mclock_ctr_songnumber;
extern unsigned char mclock_ctr_songdisplay;

extern unsigned char mclock_ctr_track1;
extern unsigned char mclock_ctr_track2;
extern unsigned char mclock_ctr_track3;
extern unsigned char mclock_ctr_track4;

extern unsigned char mclock_ctr_notetrack1;
extern unsigned char mclock_ctr_notetrack2;
extern unsigned char mclock_ctr_notetrack3;
extern unsigned char mclock_ctr_notetrack4;

extern unsigned char mclock_ctr_24;
extern unsigned char mclock_ctr_beats;
extern unsigned char mclock_ctr_measures;
extern unsigned char mclock_ctr_section;
extern unsigned char mclock_ctr_seclength;

extern unsigned char mclock_ctr_newsection;
extern unsigned char mclock_ctr_changesection;
extern unsigned char mclock_ctr_resetmeasures;
extern unsigned char mclock_ctr_resettrigger;
extern unsigned char mclock_ctr_loadseq[20];


void dosq01(void);
void dosq02(void);
void dosq03(void);
void dosq04(void);
void dosq05(void);
void dosq06(void);
void dosq07(void);
void dosq08(void);
void dosq09(void);
void dosq10(void);
void dosq11(void);
void dosq12(void);
void dosq13(void);
void dosq14(void);
void dosq15(void);
void dosq16(void);
void dosq17(void);
void dosq18(void);
void dosq19(void);
void dosq20(void);
void dosq21(void);
void dosq22(void);
void dosq23(void);
void dosq24(void);
void dosq25(void);
void dosq26(void);
void dosq27(void);
void dosq28(void);
void dosq29(void);
void dosq30(void);
void dosq31(void);
void dosq32(void);
void dosq33(void);
void dosq34(void);
void dosq35(void);
void dosq36(void);
void dosq37(void);
void dosq38(void);
void dosq39(void);
void dosq40(void);
void dosq41(void);
void dosq42(void);
void dosq43(void);
void dosq44(void);
void dosq45(void);
void dosq46(void);
void dosq47(void);
void dosq48(void);



#endif /* _MCLOCK_H */
